# -*- coding: utf-8 -*-
"""
The BaseFrame is the base class for all image-based classes.

It represents a single-layer image.
"""

import numpy
from astropy.io import fits
import tempfile

from common.database.DBMain import persistent
from micado.main.DataItem import DataItem


class BaseFrame(DataItem):
    """
    BaseFrame
    """

    # Persistent properties correspond to DataItem header keywords.
    NAXIS1: int = persistent('Length of data in axis 1 [pixel]', int, -1)
    NAXIS2: int = persistent('Length of data in axis 2 [pixel]', int, -1)

    image: numpy.ndarray = None
    # The image data

    format_time_filename = "%Y%m%dT%H%M%SS%f"
    # Format for strftime/strptime for filenames

    format_time_fitsheader = "%Y-%m-%dT%H:%M:%S.%f"
    # Format for strftime/strptime for fits headers

    def __init__(self, *args, **kwargs):
        """
        Initialize BaseFrame.
        """
        DataItem.__init__(self, *args, **kwargs)

    def make_fits_header(self):
        """
        Get FITS header.

        See Header keywords spreadsheet.
        https://docs.google.com/spreadsheets/d/1BbQkf8l8fCPSYf7skE8y0GC2UMK8kPWBkjY0ojhZqVs/edit#gid=0
        """
        return {
            'NAXIS1': self.NAXIS1,
            'NAXIS2': self.NAXIS2,
        }

    def read_fits_header(self, header):
        """
        Parse FITS header.
        """
        self.NAXIS1 = header['NAXIS1']
        self.NAXIS2 = header['NAXIS2']

    def save_fits(self, filename: str=None):
        """
        Save as FITS file.
        """
        if not filename:
            if not self.filename:
                self.filename = tempfile.NamedTemporaryFile(suffix='.fits', delete=False).name
            filename = self.filename

        hdu = fits.PrimaryHDU(self.image)
        hdu.header.update(self.make_fits_header())
        hdul = fits.HDUList([hdu])
        hdul.writeto(filename)

        self.unique_filename()

    def load_fits(self, filename: str=None):
        """
        Loads a fits file.
        """
        if filename is None:
            filename = self.filename
        else:
            self.filename = filename

        hdulist = fits.open(filename)
        self.image = hdulist[0].data
        self.read_fits_header(hdulist[0].header)

    def unique_filename(self, input_filename=None, extra=None):
        """
        Generate unique filename.
        """
        DataItem.unique_filename(
            self,
            input_filename=input_filename,
            extra=[self.creation_date.strftime(self.format_time_filename)]
        )
