# -*- coding: utf-8 -*-
"""
DataItem represents a MICADO Data Item.
"""

import os
import datetime

from common.database.DataObject import DataObject, persistent
from common.util.md5sum import md5sum
from micado.util.esoterminology import DprCategory, DprType


class DataItem(DataObject):
    """
    DataItem represents a MICADO Data Item.

    The following table describes all MICADO DataItems.
    https://docs.google.com/spreadsheets/d/1BbQkf8l8fCPSYf7skE8y0GC2UMK8kPWBkjY0ojhZqVs/edit#gid=751586500

    The columns in that table are replicated in DataItem classes as follows
    - Name:
      - self.__class__.__name__.upper()
      - "_"
      - self.__class__.__bases__[0].__name__.upper()
    - Type:
      - self.__class__.__bases__[0].__name__.upper() == "RAW" ? "DPR" : "PRO"
    - Description: self.description
    - DPR CATG: self.dpr_catg
    - DPR TECH: TODO, from class hierarchy
    - DPR TYPE: self.dpr_type
    - DO.CATG: == DPR CATG? TODO
    - PRO CATG: == DPR CATG? TODO
    - PRO TYPE: TODO, from class hierarchy
    - #Extensions: TODO, from class hierarchy?
    - Reduction Step: TODO, from class hierarchy?
    - Internal type: TODO, is this relevant?
    """

    creation_date: datetime.datetime = persistent(
        'Date this object was created [None]', datetime.datetime, datetime.datetime(1990, 1, 1))


    description = ""
    # Description, e.g. "Master dark image"

    dpr_catg = DprCategory.OTHER
    # The Category of the observation

    dpr_type = DprType.SKY
    # The type of observation

    def __init__(self, *args, **kwargs):
        """
        Initializes the DataItem.
        """
        DataObject.__init__(self, *args, **kwargs)
        self.creation_date = datetime.datetime.now()

    def unique_filename(self, input_filename: str=None, extra: list=None):
        """
        Generate unique filename.
        """
        if input_filename is None:
            input_filename = self.filename
        ext = os.path.splitext(input_filename)[1]
        classname = self.__class__.__name__.replace('_', '')
        if extra is None:
            extra = []
        hashsum = md5sum(input_filename)
        elements = [classname] + extra + [hashsum]
        self.filename = f'{"_".join(elements)}{ext}'
        if (input_filename != self.filename) and not os.path.lexists(self.filename):
            os.symlink(input_filename, self.filename)
