"""
Module with persistent Simulator classes.

These classes are not used anymore and will be refactored into
- classes modeling the instrument / universe /etc
- classes specifically for interfacing with SimCADO
"""

import socket

from datetime import datetime

from common.database.DBMain import DBObject, persistent
from common.database.DataObject import DataObject
from common.database.DBSelect import Select


class SimBase(object):
    """
    Base class for all Simulator related input classes.
    """

    def __init__(self):
        """
        SimBase constructor.
        """

        self.filename: str = None
        # Name of file with data corresponding to this object.


# TRANSMISSION CLASSES


class SimTransmissionCurve(DataObject, SimBase):
    """SimTransmissionCurve"""
    ...


class SimAtmosphericTC(SimTransmissionCurve):
    """SimAtmosphericTC"""
    ...


class SimADCTC(SimTransmissionCurve):
    """SimADCTC"""
    ...


class SimDichroicTC(SimTransmissionCurve):
    """SimDichroicTC"""
    ...


class SimEntranceWindowTC(SimTransmissionCurve):
    """SimEntranceWindowTC"""
    ...


class SimFilterTC(SimTransmissionCurve):
    """SimFilterTC"""
    ...


class SimMirrorTC(SimTransmissionCurve):
    """SimMirrorTC"""
    ...

# TODO remove
# class SimSkyCalcCurve(DataObject):
#    pass


# DETECTOR CLASSES

class SimDetector(DataObject, SimBase):
    """SimDetector"""
    ...


class SimChipLayout(SimDetector):
    """SimChipLayout"""
    ...


class SimNoiseFrame(SimDetector):
    """SimNoiseFrame"""
    ...


class SimQE(SimDetector):
    """SimQE"""
    ...


class SimPCA(SimDetector):
    """SimPCA"""
    ...


# INPUT CLASSES

class SimPsf(DataObject, SimBase):
    """SimPsf"""
    ...


class SimSources(DataObject, SimBase):
    """SimSources"""
    ...

# class SimScienceObject(DataObject):
#    pass


# PARAMETER CLASSES

class SimulatorParameters(DBObject):
    """SimulatorParameters"""
    ...
    # xxx = persistent('process parameter x [None]', str, '')
    # yyy = persistent('process parameter y [None]', int, 0)


class ParameterNameValue(DBObject):
    """ParameterNameValue"""
    name: str = persistent('name of the parameter [None]', str, '')
    value: str = persistent('value of the parameter [None]', str, '')
    comment: str = persistent('comment of the parameter [None]', str, '')
    # TODO is default value needed to store ?
    # default = persistent('default value of the parameter [None]', str, '')


# SIMULATOR CLASS

class SimulatorRun(DBObject):
    """SimulatorRun"""

    # mapping config items to class and attributes
    config_class = {
        # ATMO_EC   Atmospheric emission curve
        'ATMO_TC': (SimAtmosphericTC, 'atmospheric_tc'),  # Atmospheric transmission curve
        'FPA_CHIP_LAYOUT': (SimChipLayout, 'chip_layout'),  # description of the chip layout on the detector array
        'FPA_NOISE_PATH': (SimNoiseFrame, 'noise_frame'),  # noise frame
        'FPA_QE': (SimQE, 'quantum_eff'),  # FPA quantum efficiency
        'HXRG_PCA0_FILENAME': (SimPCA, 'pca'),  # principal component analysis
        'INST_ADC_TC': (SimADCTC, 'adc_tc'),  # transmission curve of the ADC
        'INST_DICHROIC_TC': (SimDichroicTC, 'dichroic_tc'),  # dichroic TC
        'INST_ENTR_WINDOW_TC': (SimEntranceWindowTC, 'entrance_win_tc'),  # Entrance Window TC
        'INST_FILTER_TC': (SimFilterTC, 'filter_tc'),  # broadband filter TC
        'SCOPE_M1_TC': (SimMirrorTC, 'mirror_tc'),  # Mirror reflectance curve
        'SCOPE_PSF_FILE': (SimPsf, 'psf'),  # The PSF
    }

    # Attributes of the actual simulation run (simple types)
    version: str = persistent('Version of the simulator [None]', str, '')
    run_date: datetime = persistent(
        'The data the simulator was run [None]', datetime, datetime(1990, 1, 1))
    run_duration: float = persistent(
        'The duration of the simulator run [s]', float, float('NaN'))
    machine_dns: str = persistent(
        'The machine the simultaro was run on [None]', str, '')

    # Inputs
    sources: SimSources = persistent(
        'The input source [None]', SimSources, None)
    atmospheric_tc: SimAtmosphericTC = persistent(
        'Atmospheric transmission curve [None]', SimAtmosphericTC, None)
    chip_layout: SimChipLayout = persistent(
        'the chip layout on the detector array [None]', SimChipLayout, None)
    noise_frame: SimNoiseFrame = persistent(
        'noise frame [None]', SimNoiseFrame, None)
    quantum_eff: SimQE = persistent(
        'FPA quantum efficiency [None]', SimQE, None)
    pca: SimPCA = persistent(
        'principal component analysis [None]', SimPCA, None)
    adc_tc: SimADCTC = persistent(
        'transmission curve of the ADC [None]', SimADCTC, None)
    dichroic_tc: SimDichroicTC = persistent(
        'transmission curve of the dichroic [None]', SimDichroicTC, None)
    entrance_win_tc: SimEntranceWindowTC = persistent(
        'transmission curve of the Entrance Window [None]', SimEntranceWindowTC, None)
    filter_tc: SimFilterTC = persistent(
        'transmission curve of the (broadband) filter [None]', SimFilterTC, None)
    mirror_tc: SimMirrorTC = persistent('Mirror (1) reflectance curve [None]', SimMirrorTC, None)
    psf: SimPsf = persistent('The input PSF [None]', SimPsf, None)

    # process parameter objects
    process_parameters: SimulatorParameters = persistent(
        'The simulator process parameters [None]', SimulatorParameters, None)
    process_parameters_nv: ParameterNameValue = persistent(
        'List of all process paramerers as name-value pairs [None]', ParameterNameValue, [])

    def make(self):
        """
        Make a SimulatorRun object.
        """
        # TODO run simcado ?
        raise NotImplementedError('SimulatorRun.make method not yet implemented')

    def init_from_config(self, config, commit=False):
        """
        Initialize from a SimCADO config.
        """
        # TODO version and run_duration
        self.run_date = datetime.now()
        self.machine_dns = socket.getfqdn()
        for name, value in config.items():
            nv = ParameterNameValue()
            nv.name = name
            nv.value = str(value)
            self.process_parameters_nv.append(nv)
            if name in self.config_class:
                attr_cls, attr_name = self.config_class[name]
                attr = attr_cls()
                attr.unique_filename(value)
                query: Select = attr_cls.filename == attr.filename
                if len(query) > 0:
                    setattr(self, attr_name, query.max('+CREATED'))
                else:
                    setattr(self, attr_name, attr)
                    if commit:
                        getattr(self, attr_name).store()
